// ---------------------------------------------------------------------------
// Created by F. Malpartida on 9/9/12.
// Copyright 2011 - Under creative commons license 3.0:
//        Attribution-NonCommercial-ShareAlike CC BY-NC-SA
//        http://www.electrofunltd.com
// 
// This software is furnished "as is", without technical support, and with no 
// warranty, express or implied, as to its usefulness for any purpose.
//
// Thread Safe: No
// Extendable: No
//
// @file mmi.cpp
// Main application MMI header files. 
// 
// @brief 
// Requires New LiquidCrystal library to be installed in system:
//    https://bitbucket.org/fmalpartida/new-liquidcrystal/wiki/Home
// Renders to an LCD module the main status of the CNC controller: axis position
// feed rates, on board temperature, fan speeds.
// MMI availability can be compiled out from the system by undifining the _MMI_ENABLED_
// flag from the configMe.h header file.
//
// @see mmi.h for class method documentation
//
//
// @author F. Malpartida
// ---------------------------------------------------------------------------
#include "mmi.h"

#ifdef _MMI_ENABLED_
#include <LCD.h>
#include "tempCntrl.h"
#include "stepperCntrl.h"


#define CHAR_WIDTH   5   // defined for bargraph display

// Extenal variables with code location information
extern unsigned int __bss_end;
extern unsigned int __heap_start;
extern void *__brkval;

/*!
 @function
 @abstract   Return available RAM memory
 @discussion This routine returns the ammount of RAM memory available after
 initialising the C runtime.
 @param
 @result     Free RAM available.
 */
static int freeMemory()
{
   int free_memory;
   
   if((int)__brkval == 0)
      free_memory = ((int)&free_memory) - ((int)&__bss_end);
   else
      free_memory = ((int)&free_memory) - ((int)__brkval);
   
   return free_memory;
}

/*!
 @function
 @abstract   Load bitmaps into the LCD
 @discussion This loads into the LCD a predefined set of new characters
 from position 0.
 @param      myLCD: LCD object in which to load the character map.
 @result     Free RAM available.
 */
static void loadCharMap ( LCD *myLCD )
{
   /*!
    @const      charBitmap
    @abstract   Define Character bitmap for the bargraph.
    @discussion Defines a character bitmap to represent a bargraph on a text
    display. The bitmap goes from a blank character to full black.
    */
   const uint8_t charBitmap[][8] = {
      { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 },
      { 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x10, 0x0 },
      { 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x18, 0x0 },
      { 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x1C, 0x0 },
      { 0x1E, 0x1E, 0x1E, 0x1E, 0x1E, 0x1E, 0x1E, 0x0 },
      { 0x1F, 0x1F, 0x1F, 0x1F, 0x1F, 0x1F, 0x1F, 0x0 },
      { 0xe, 0x11, 0x11, 0x11, 0xe, 0, 0, 0 },
      { 0x6, 0x9, 0x9, 0x6, 0x0, 0, 0, 0}
   };
   
   uint8_t charBitmapSize = (sizeof(charBitmap ) / sizeof (charBitmap[0]));
   
   for ( uint8_t i = 0; i < charBitmapSize; i++ )
   {
      myLCD->createChar ( i, (uint8_t *)charBitmap[i] );
   }
   
}

/*!
 @function
 @abstract   Print splash screen
 @discussion Prints the powerup splash screen for 2 seconds 
 @param      myLCD: LCD object where to print the Splash screen.
 @result
 */
static void printSplash ( LCD *lcd )
{
   // Print splash screen
   lcd->setCursor ( 0, 0 );
   lcd->print ( F("Firmware: ELFCNC") );
   lcd->setCursor ( 0, 1 );
   lcd->print ( F("Version: ") );
   lcd->print ( FRMW_VERSION );
   delay ( 2000 );
   lcd->clear ();
#ifdef _DEBUG_
   lcd->print ( F("Free mem:"));
   lcd->print ( freeMemory() );
   delay ( 2000 );
#endif
}

static void drawBars ( LCD *myLCD, int value, uint8_t col, uint8_t row, 
                       uint8_t barLength, char start, char end, const char title[] )
{
   int numBars;
   
   // Set initial titles on the display
   myLCD->setCursor (col, row);
   myLCD->print ( title );
   myLCD->print (start);
   
   // Calculate the size of the bar
   value = map ( value, 0, 255, 0, ( barLength ) * CHAR_WIDTH );
   numBars = value / CHAR_WIDTH;
   
   // Limit the size of the bargraph to barLength
   if ( numBars > barLength )
   {
      numBars = barLength;
   }
   myLCD->setCursor ( col + strlen(title) + 1, row );
   
   // Draw the bars
   while ( numBars-- )
   {
      myLCD->print ( char( 5 ) );
   }
   
   // Draw the fractions
   numBars = value % CHAR_WIDTH;
   myLCD->print ( char(numBars) );
   myLCD->setCursor (col + barLength + strlen(title) + 1, row);
   myLCD->print (end);
}


// CLASS CONSTRUCTORS
// ---------------------------------------------------------------------------
// Constructor
mmi::mmi ( void )
{
   _lcd       = NULL;
#ifdef _TEMP_CONTROL_ENABLED_
   _tempCont  = NULL;
#endif
   _machCntrl = NULL;
   _init      = false;
}

// PUBLIC CLASS METHODS
// ---------------------------------------------------------------------------
// Initialization methods
#ifdef _TEMP_CONTROL_ENABLED_
void mmi::init ( LCD *lcd, tempCntrl *temperatureController, stepperCntrl *controller )
{
   _lcd       = lcd;
   _tempCont  = temperatureController;
   _machCntrl = controller;
   _init      = true;
   
   loadCharMap ( lcd );
   printSplash ( lcd );
   
}
#else
void mmi::init ( LCD *lcd, stepperCntrl *controller )
{
   _lcd       = lcd;
   _machCntrl = controller;
   _init      = true;
   
   loadCharMap ( lcd );
   printSplash ( lcd );
}
#endif // _TEMP_CONTROL_ENABLED_

// LCD rendering
void mmi::display ( void )
{
   uint8_t                    speed;
   stepperCntrl::t_floatPoint pos;
   static uint8_t             counter = 0;

   if (counter >= 8 )
   {
     counter = 0;
     _lcd->clear ();
   }
   else
   {
     counter++;
   }
   
#ifdef _TEMP_CONTROL_ENABLED_  
   speed = _tempCont->speedGet();
   
   // Read the current temperature form the sensor
   _lcd->setCursor ( 0,0 );
   _lcd->print ( "T:" );
   _lcd->print ( _tempCont->tempGet(), 1 );
   
   drawBars ( _lcd, speed, 7, 0, 5, '[', ']', "Fn" );
#endif
   
   pos = _machCntrl->getPosition ();

   _lcd->setCursor ( 0, 1 );
   if ( counter < 4 )
   {
      _lcd->print ("X:");
      _lcd->print (pos.x, 2);
      _lcd->print (" ");
      _lcd->print ("Y:");
      _lcd->print (pos.y, 2);
   }
   else
   {
      _lcd->print ("Z:");
      _lcd->print (pos.z, 2);
      _lcd->print (" ");
      _lcd->print ("FR:");
      _lcd->print (_machCntrl->getFeedRate());
   }
   
   
#ifdef _DEBUG_
#ifdef _TEMP_CONTROL_ENABLED_
   Serial.print ( _tempCont->tempGet(), 1 );
   Serial.println ( speed  );
#endif // _TEMP_CONTROL_ENABLED_
   Serial.print ( freeMemory() );
   Serial.print (pos.x, 1);
   Serial.print (" ");
   Serial.print (pos.y, 1);
   Serial.print (" ");
   Serial.println (pos.z, 1);
#endif // _DEBUG_
}
#endif // _MMI_ENABLED_
