// ---------------------------------------------------------------------------
// Created by F. Malpartida on 9/9/12.
// Copyright 2011 - Under creative commons license 3.0:
//        Attribution-NonCommercial-ShareAlike CC BY-NC-SA
//        http://www.electrofunltd.com
//
// This software is furnished "as is", without technical support, and with no
// warranty, express or implied, as to its usefulness for any purpose.
//
// Thread Safe: No
// Extendable: No
//
// @file stepperCntrl.cpp
// Main machine controller.
//
// @brief This class does all the coordinated movement of the stepper motors in
// terms of moving the three axis to a given coordinate. This class controls
// the linear movements, controls the machine's coordinates and positioning,
// axis velocity.
//
// Acknowledgements:
// Inspired on the work of Carlos guilarte y diego colonnello (Txapuzas CNC).
//
// @author F. Malpartida
// ---------------------------------------------------------------------------
#ifndef _stepperCntrl_h_
#define _stepperCntrl_h_

#include "configMe.h"
#include "stepper.h"

class stepperCntrl
{
   // Point structures
   /*!
    @typedef
    @abstract   define point structures
    @discussion point structure definitions used to hold the steps on each axis
    @field      x - x axis
    @field      y - y axis
    @field      z - z axis
    */
   typedef struct LongPoint
   {
      long x;
      long y;
      long z;
   } t_longPoint;
   
   /*!
    @typedef
    @abstract   define point structures
    @discussion point structure definitions used to hold the direction on each axis
    @field      x - x axis
    @field      y - y axis
    @field      z - z axis
    */
   typedef struct u8Point
   {
      uint8_t x;
      uint8_t y;
      uint8_t z;
   } t_u8Point;
   
public:
   
   /*!
    @typedef
    @abstract   define point structures
    @discussion point structure definitions used to hold coordinates in mm
    @field      x - x axis
    @field      y - y axis
    @field      z - z axis
    */
   typedef struct FloatPoint
   {
      float x;
      float y;
      float z;
   } t_floatPoint;
   
   /*!
    @function
    @abstract   class constructor, initializes all data structures and creates
    the stepper motors.
    @discussion Class constructor. Creates the machine controller and associates
    the stepper motors. To change pin allocation and mapping please use the
    "configMe.h" file.
    */
   stepperCntrl ( void );
   
   /*!
    @function
    @abstract   Initialises the machine controller.
    @discussion Initialises machine controller. In this case, it configures the
    feedrate delay. This routine has to be called prior to calling any method
    of this class.
    */
   void init (void );
   
   // Coordinate and target locations
   /*!
    @function
    @abstract   Sets the new target position for the machine.
    @discussion Sets the new target in the machine. If also uses the current
    position to compute the delta movement that the machine has to make and
    the associated feed rate delay. At this point the machine is ready to move.
    @param      x axis in mm or inch
    @param      y axis in mm or inch
    @param      z axis in mm or inch
    */
   void setTarget(float x, float y, float z);
   
   /*!
    @function
    @abstract   Sets the current position of the machine.
    @discussion Sets the current position of the machine, it also recalculates
    any delta movement, should the target position differ.
    @param      x axis in mm or inch
    @param      y axis in mm or inch
    @param      z axis in mm or inch
    */
   void setPosition(float x, float y, float z);
   
   /*!
    @function
    @abstract   Returns the current machine position.
    @discussion Returns the current machine position expressed in the configured
    units. Default units - mm
    @result     x, y, z - position in mm or inch.
    */
   t_floatPoint getPosition ( void );
   
   /*!
    @function
    @abstract   Computes the delta to move from the current position to the current
    target.
    @discussion computes all the deltas that are necessary to move the machine
    from the current position to the targe position. This routine will compute the
    movement and its direction that the machine will have to make.
    */
   void  computeDeltas (void);
   
   /*!
    @function
    @abstract   Sets the machine working units.
    @discussion Sets the machine working units expressed relative to mm. Therefore,
    to set inches - 2.54 will be passed.
    @param      units magnitude of the new working units expressed in mm (2.54 to
    work in inches).
    */
   void  setUnits (float units);
   
   /*!
    @function
    @abstract   Set the curve section.
    @discussion Sets the curve section of the machine. The value is take from
    the configMe.h CURVE_SECTION_MM.
    @param      curveSection curve section in mm.
    @see        CURVE_SECTION_MM in configMe.h
    */
   void  setCurveSection ( float curveSection );
   
   /*!
    @function
    @abstract   Gets the currently configured curve section.
    @discussion Gets the currently configured curve section.
    @result     curveSection curve section in the currently configured magnitude.
    */
   float getCurveSection ( );
   
   /*!
    @function
    @abstract   Sets the coordinate working mode to be absolute.
    @discussion Sets the current coordinate working mode to be absolute. All movements
    are specified from the origin.
    */
   void  setAbsoluteCoord ( void );
   
   /*!
    @function
    @abstract   Sets the coordinate working mode to be relative.
    @discussion Sets the current coordinate working mode to be relative to the
    current position. All movements are specified from the current position.
    */
   void  setRelativeCoord ( void );
   
   /*!
    @function
    @abstract   Returns true if working in absolute coordinate mode.
    @result     true if working in absolute coordinate mode, false in any other case.
    */
   bool  absoluteCoord ( void );
   
   
   // Feed rate and movement velocity methods
   
   /*!
    @function
    @abstract   Returns the machine maximum speed in mm per minute.
    @result     Returns the machine's maximum speed capabibility in mm per min.
    */
   long  getMaxSpeed( void );
   
   /*!
    @function
    @abstract   Set the machine feed rate (movement speed)
    @param      Speed rate at which the machine will move in mm per minute.
    */
   void  setFeedRate ( long feedDelay );
   
   /*!
    @function
    @abstract   Get the machine feed rate (movement speed)
    @param      Speed rate at which the machine will move in mm per minute.
    */
   long getFeedRate ( void );
   
   // Machine movement methods
   /*!
    @function
    @abstract   Move the machine from the current position to the target position
    in a straight line.
    @discussion Move all three axis from the current position in _current_units
    to position _target_units. This routine in normally called after invoking setTarget.
    The machine will perform a linear movement using the dda algorithm.
    */
   void  moveLine ( void );
   
   /*!
    @function
    @abstract   Move the machine from the current position to the target position
    in an arch.
    @discussion Move the two axis from the current position in _current_units
    to position fp using i,j as the circunference center in a CW or ACW movement.
    @param      fp destination point (it will only process X, Y movements - Z is
    omitted.
    @param      i - x coordinate of the center radius
    @param      j - y coordinate of the center radius
    @param      cwMove if true, movement will be performed clockwise, if false
    the movement will be anticlockwise.
    */
   void  moveArc ( t_floatPoint fp, double i, double j, bool cwMove );
   
   // Stepper driver control methods.
   
   /*!
    @function
    @abstract   Disable all the stepper drivers in the machine.
    */
   void  disableSteppers ();
   
   /*!
    @function
    @abstract   Enables all the stepper drivers in the machine.
    */
   void  enableSteppers ();
   
   
private:
   // Convertion methods, positioning and movement methods
   long  toSteps ( float stepsPerUnit, float units );
   
   bool canStep (uint8_t min_pin, uint8_t max_pin, long current, long target,
                 uint8_t direction);
   
   // Read end-stops
   bool  readSwitch(uint8_t pin);
   
   // Machine time and feedrate methods
   long  calculateFeedrateDelay(long feedrate);
   void  setFeedDelay ( );
   
   //our feedrate variables.
   long  _feedRate;              // Feed rate expressed in mm/min
   long  _feedrateDelay;         // Feed rate delay - time take to move to target base on feedrate
   long  _lastSpeed;             // Last speed set
   
   
   float       _units_in_mm;     // Units expressed in mm. 1 inch 25.4 mm
   t_longPoint _stepsPerUnit;    // Number of steps per configured units
   t_u8Point   _direction;       // Movement direction for each axis - 0 or 1
   
   t_floatPoint _current_units;  // Current position expressed in current units (mm or inch)
   t_floatPoint _target_units;   // Target position expressed in current units (mm or inch)
   t_floatPoint _delta_units;    // Difference between current position and target position
   
   t_longPoint _current_steps;   // Current position expressed in steps
   t_longPoint _target_steps;    // Target position expressed in steps
   t_longPoint _delta_steps;     // Difference between current and target position
   
   float _curveSection;          // Curve section
   
   stepper *_xStepper;           // Stepper driver controlling X axis.
   stepper *_yStepper;           // Stepper driver controlling Y axis.
   stepper *_zStepper;           // Stepper driver controlling Z axis.
   
   bool _absMode;
   unsigned long _lastStepTime;  // Time in micro-seconds since the last step
   
};

#endif